@@ -17,6 +17,7 @@
 #' @param entry a numeric value indicating the enrollment time, Default: 1
 #' @param fup a numeric value indicating the minimum follow-up time for subjects.
 #'  , Default: 1
+#' @param maxfup maximum follow-up time
 #' @param CtrlHaz a function,  specifying the hazard function for control group.
 #' @param hazR a function, specifying the hazard ratio function between
 #' treatment and control group
@@ -34,9 +35,6 @@
 #' @param entry_pdf0 a function, indicating the probability density function (pdf)
 #' of enrollment time for control group. The default assumes a uniform distribution
 #' corresponding to the constant enrollment rate.
-#' Default: function(x) {
-#'    (1/entry) * (x >= 0 & x <= entry)
-#'}
 #' @param entry_pdf1 a pdf of enrollment time for treatment
 #' group. See \code{entry_pdf0}, Default: assume same pdf as control group.
 #' @param ratio an integer, indicating the randomization ratio between treatment
@@ -51,6 +49,15 @@
 #' obtaining the number of events. See details , Default: 500
 #' @param k an integer, indicating number of sub-intervals per time unit,
 #'  Default: 100
+#' @param m a value within 0 and 1.
+#' @param nreps an integer, indicating number of iterations in calculating
+#' the quantile of multivariate normal. See Details.
+#' @param varianceType Default: \code{equal}. Indicates different variance assumptions
+#'  for the sample size calculation.
+#' It is not applicable for the maximum weighted logrank test. See details.
+#' @param weightBase A character, either "\code{F}" or "\code{T}". \code{F} indicates
+#' a CDF is the base for the weight function used in the weighted logrank or maximum weighted
+#' logrank test. \code{T} indicates time is the base for weight function. Default: \code{F}
 #' @param summary a logical value, controlling whether to print the
 #' summary of calculation, Default: TRUE
 #' @return
@@ -60,6 +67,8 @@
 #'  \item{totalN}{total number of subjects}
 #'  \item{pwr}{actual power given the number of events}
 #'  \item{prob_event}{event probability at the end of trial}
+#'  \item{prob1}{event probability for the treatment group}
+#'  \item{prob0}{event probability for the control group}
 #'  \item{L_trans}{a list, consisting of transition matrix at each interval}
 #'  \item{pdat}{ a dataframe including all the intermediate variables in the calculation.
 #'  see Details.}
@@ -87,7 +96,8 @@
 #'test or combination test is implemented. An iterative procedure
 #'is used to obtain the event number based on the multivariate normal distribution.  Package
 #'\pkg{mvtnorm} is used to calculate the quantiles. Because the algorithm is slightly
-#'seed dependent, the quantiles are mean values of ten replicates.
+#'seed dependent, the quantiles are mean values of ten replicates by default. The
+#'number of replicates is controlled by argument \code{ninter}.
 #'
 #'The "\code{alternative}" option supports both two-sided and one-sided test.
 #' Let \eqn{\Lambda_1} and \eqn{\Lambda_0} denote the cumulative hazard of
@@ -96,6 +106,12 @@
 #' \eqn{H_a: \Lambda_1 <= \Lambda_0}. The \code{greater} option tests
 #'\eqn{H_0: \Lambda_1 < \Lambda_0} against \eqn{H_a: \Lambda_1 >= \Lambda_0}.
 #'
+#'When \code{varianceType} is \code{equal}, the sample size for a two sided test is
+#'\eqn{(z_{1-\alpha/2}+z_{1-\beta})^2\tilde{\sigma}^2/\mu_w^2}, where \eqn{\tilde{\sigma}^2}
+#' is the variance estimate under alternative. when \code{varianceType} is not
+#' \code{equal}. The formula is \eqn{(z_{1-\alpha/2}\sigma_w+z_{1-\beta}\tilde{\sigma})^2/\mu_w^2}.
+#' Please use \code{equal} variance type for the maximum weighted logrank test.
+#'
 #' @references
 #'
 #' Brendel, M., Janssen, A., Mayer, C. D., & Pauly, M. (2014). Weighted logrank
@@ -106,8 +122,9 @@
 #' Crossing Hazards. arXiv preprint arXiv:2110.03833.
 #'
 #'
-#' Cheng, H., & He, J. (2021). Sample size calculation for the maximum weighted
-#'  logrank test under non-proportional hazards (to submit)
+#' Cheng H, He J. Sample size calculation for the combination test under
+#' nonproportional hazards. Biom J. 2023 Apr;65(4):e2100403. doi: 10.1002/bimj.202100403.
+#' Epub 2023 Feb 15. PMID: 36789566
 #' @examples
 #'  #------------------------------------------------------------
 #'  ## Delayed treatment effects using maxcombo test
@@ -148,13 +165,13 @@
 #'  \code{\link{gen.wgt}}, \code{\link{evalfup}}
 #' @rdname pwr2n.NPH
 #' @export
-#' @importFrom stats cov2cor weighted.mean
+#' @importFrom stats cov2cor weighted.mean na.omit
 #' @importFrom mvtnorm qmvnorm pmvnorm
 
-###input parameters
 pwr2n.NPH<- function(method = "MaxLR"
                      ,entry   = 1
                      ,fup      = 1
+                     ,maxfup = entry+fup
                      ,CtrlHaz
                      ,hazR
                      ,transP1 = c(0, 0)
@@ -168,7 +185,12 @@ pwr2n.NPH<- function(method = "MaxLR"
                      ,alternative = c("two.sided")
                      ,criteria = 500
                      ,k        = 100
+                     ,m = 0
+                     ,nreps = 10
+                     ,varianceType = c("equal") # not applicable for combination test
+                     ,weightBase = "C" # C for CDF ； T for time
                      ,summary = TRUE
+
 ){
 
   if (!method %in% c("MaxLR","Projection")){
@@ -178,13 +200,19 @@ pwr2n.NPH<- function(method = "MaxLR"
     stop("The 'alternative' must be one of 'two.sided','greater','less'.")
   }
 
-  tot_time <- entry+fup
+  tot_time <- maxfup
   num <- k*tot_time
   # create the subintervals
-  x <- seq(0,tot_time,by=1/k)
+  if (m >1 |m <0){stop("m must be within 0 and 1")}
+  x <- seq(1/k*m,tot_time,by=1/k)
   # if control hazard is zero at x=0, then use a small value for x like
   # 0.0001
-  if (CtrlHaz(0)==0){x[1] <- 0.0001}
+  ## 2022-01-19 or control hazard is Inf
+  if (m==0){
+    if (CtrlHaz(0)==0|CtrlHaz(0)==Inf|hazR(0)==0|hazR(0)==Inf){
+      x[1] <- 1/k*0.1}
+  }
+
   ctrlRate <- CtrlHaz(x)
   haz_val <- hazR(x)*ctrlRate
   haz_point <- x*k
@@ -193,27 +221,52 @@ pwr2n.NPH<- function(method = "MaxLR"
                     haz_point=haz_point,ratio=ratio,
                     transP1=transP1,transP0=transP0,k=k,
                     fup=fup,entry=entry,entry_pdf0=entry_pdf0,
-                    entry_pdf1=entry_pdf1)
+                    entry_pdf1=entry_pdf1,hazR=hazR,tot_time=tot_time)
 
-  pdat <- load$pdat
+  old.nrow <- nrow(load$pdat)
+  pdat <- na.omit(load$pdat)
+  if (nrow(pdat)<old.nrow){
+    message(paste("At least",old.nrow-nrow(pdat), "NAs are produced in the transition matrix. Please Check
+                  whether initial hazards are zero."))
+    num <- nrow(pdat)
+  }
   wn <- length(Wlist)
   W <- matrix(NA,nrow=nrow(pdat),ncol=wn)
+  # print("step1:")
   ## calculate the variance-covariance matrix
   Vmat <- matrix(NA,nrow=wn,ncol=wn)
   event <- c()
   if (alternative=="two.sided"){
-    part2=(qnorm(1-alpha/2)+qnorm(1-beta))^2
+    div <- 2
+    #part2=(qnorm(1-alpha/2)+qnorm(1-beta))^2
   }else {
-    part2=(qnorm(1-alpha)+qnorm(1-beta))^2
+    #part2=(qnorm(1-alpha)+qnorm(1-beta))^2
+    div <- 1
   }
+  part2=(qnorm(1-alpha/div)+qnorm(1-beta))^2
   for (j in 1:wn){
-    W[,j] <- Wlist[[j]](1-pdat$S) # to be consistent with MWLR test
+    if (weightBase == "C"){
+      W[,j] <- Wlist[[j]](1-pdat$S) # to be consistent with MWLR test
+    } else if (weightBase == "T"){
+      W[,j] <- Wlist[[j]](pdat$ti)
+    }
+
     wgt <- W[,j]
-    part1=t(pdat$rho)%*%(pdat$eta*wgt^2)
+
     part3=t(pdat$rho)%*%(pdat$gamma*wgt)
     part3=part3^2
-    event[j] <- part1*part2/part3
+    if (varianceType=="equal"){
+      part1=t(pdat$rho)%*%(pdat$eta*wgt^2)
+      event[j] <- part1*part2/part3
+    } else{
+      # different variance for the sample size
+      pdat$eta_new <- ratio/(ratio+1)^2
+      part1_1 <- t(pdat$rho)%*%(pdat$eta*wgt^2)
+      part1_2 <- t(pdat$rho)%*%(pdat$eta_new*wgt^2)
+      event[j] <- (qnorm(1-alpha/div)*sqrt(part1_2)+qnorm(1-beta)*sqrt(part1_1))^2/part3
+    }
   }
+  #print("step2:")
   #print(c(part1,part2,part3))
   ## get the min and max sample size
   E_min=min(event)
@@ -239,26 +292,26 @@ pwr2n.NPH<- function(method = "MaxLR"
           power <- 1-mvtnorm::pmvnorm(-crit,crit,mean=mu,sigma = rho_est)
           return(c(crit,power))
         }
-        ftwod <- apply(do.call(rbind,sapply(1:10,ftwo,simplify=FALSE)) ,2,mean)
+        ftwod <- apply(do.call(rbind,sapply(1:nreps,ftwo,simplify=FALSE)) ,2,mean)
         power <- ftwod[2]
         crit <- ftwod[1]
       }else if (alternative=="less"){ # l1 <l0
         ftwo <- function(i){
           # set.seed(i)
-          crit <- mvtnorm::qmvnorm(alpha,tail="lower.tail",mean=rep(0,wn),sigma = rho_est)$quantile
+          crit <- mvtnorm::qmvnorm(1-alpha,tail="upper.tail",mean=rep(0,wn),sigma = rho_est)$quantile
           power <- 1-mvtnorm::pmvnorm(crit,Inf,mean=mu,sigma = rho_est)[1]
           return(c(crit,power))
         }
-        ftwod <- apply(do.call(rbind,sapply(1:10,ftwo,simplify=FALSE)) ,2,mean)
+        ftwod <- apply(do.call(rbind,sapply(1:nreps,ftwo,simplify=FALSE)) ,2,mean)
         crit <- ftwod[1];  power <- ftwod[2]
       }else if (alternative=="greater"){
         ftwo <- function(i){
           #set.seed(i)
-          crit <- qmvnorm(alpha,tail="upper.tail",mean=rep(0,wn),sigma = rho_est)$quantile
+          crit <- qmvnorm(1-alpha,tail="lower.tail",mean=rep(0,wn),sigma = rho_est)$quantile
           power <- 1-pmvnorm(-Inf,crit,mean=mu,sigma = rho_est)[1]
           return(c(crit,power))
         }
-        ftwod <-  apply(do.call(rbind,sapply(1:10,ftwo,simplify=FALSE)) ,2,mean)
+        ftwod <-  apply(do.call(rbind,sapply(1:nreps,ftwo,simplify=FALSE)) ,2,mean)
         crit <- ftwod[1];  power <- ftwod[2]
       }
     }
@@ -285,10 +338,12 @@ pwr2n.NPH<- function(method = "MaxLR"
     else {break}
 
   }
-
+  #print("step3:")
+  #print(c(event, power,dnum, wn))
   eprob <- stats::weighted.mean(c(pdat$C_E[num],pdat$E_E[num]),w=c(1,ratio))
   Nsize <- dnum/eprob
-
+  ## 2022-01-19
+  eprob1 <- pdat$E_E[num]; eprob0 <- pdat$C_E[num]
   inparam <- c("Method","Entry Time", "Follow-up Time",
                "Allocation Ratio", "Type I Error", "Type II Error",
                "Alternative","Number of Weights")
@@ -309,7 +364,7 @@ pwr2n.NPH<- function(method = "MaxLR"
     print(outputdata, row.names = FALSE)
   }
   summaryoutput <- list(input = inputdata, output = outputdata)
-
+  # print("step4:")
   inputfun <-list(
     alpha = alpha,
     beta = beta,
@@ -324,10 +379,13 @@ pwr2n.NPH<- function(method = "MaxLR"
     Weightfunctions = Wlist
   )
   names(event) <- paste0("w", 1:wn)
+  #print("step5:")
   listall <- list( eventN  = dnum
                    ,totalN = Nsize
                    ,pwr = as.numeric(power)
                    ,prob_event =eprob
+                   ,prob1 = eprob1
+                   ,prob0 = eprob0
                    ,L_trans = load$L_trans
                    ,pdat = pdat
                    ,studytime=c(entry,fup)
@@ -338,10 +396,37 @@ pwr2n.NPH<- function(method = "MaxLR"
 
   )
   class(listall) <-"NPHpwr"
+  #print("step6:")
   return(listall)
 
 
 }
+
+summary.NPHpwr <- function(object,...){
+  summary <- object$summaryout
+  wl  <- lapply(object$inputfun$Weightfunctions,deparse)
+  wlc <- paste0(noquote(unlist(noquote(lapply(wl,"[",3)))),collapse=";")
+  wlc <- gsub("\\s","",wlc)
+  input <- data.frame(Parameter = c(summary$input$parameter,"Weight Functions"),
+                      Value = c(summary$input$value, wlc))
+
+  output <- summary$output
+  names(input) <- c("__Parameter__", "__Value__")
+  names(output) <- c("__Parameter__", "__Value__")
+  cat("------------------------------------------ \n ")
+  cat("----Summary of the Input Parameters---- \n")
+  cat("------------------------------------------ \n ")
+
+  print(input, row.names = FALSE, right=FALSE, justify = "left")
+  cat("------------------------------------------ \n ")
+  cat("-----Summary of the Output Parameters----- \n ")
+  cat("------------------------------------------ \n ")
+  print(output, row.names = FALSE, right=FALSE,  justify = "left")
+  cat("------------------------------------------ \n ")
+
+
+   cat("Notes: the base (x) of weight function is \n K-M estimate of CDF ")
+}
 #' @title Summary of the \code{pwr2n.NPH} function
 #' @description Summarize and print the results of \code{pwr2n.NPH} function
 #' @param object object of the \code{pwr2n.NPH} function
@@ -353,17 +438,17 @@ pwr2n.NPH<- function(method = "MaxLR"
 #' @rdname summary.NPHpwr
 #' @export
 summary.NPHpwr <- function(object,...){
-   summary <- object$summaryout
-   wl  <- lapply(object$inputfun$Weightfunctions,deparse)
-   wlc <- paste0(noquote(unlist(noquote(lapply(wl,"[",3)))),collapse=";")
-   wlc <- gsub("\\s","",wlc)
-   input <- data.frame(Parameter = c(summary$input$parameter,"Weight Functions"),
-                       Value = c(summary$input$value, wlc))
+  summary <- object$summaryout
+  wl  <- lapply(object$inputfun$Weightfunctions,deparse)
+  wlc <- paste0(noquote(unlist(noquote(lapply(wl,"[",3)))),collapse=";")
+  wlc <- gsub("\\s","",wlc)
+  input <- data.frame(Parameter = c(summary$input$parameter,"Weight Functions"),
+                      Value = c(summary$input$value, wlc))
 
-   output <- summary$output
-   names(input) <- c("__Parameter__", "__Value__")
-   names(output) <- c("__Parameter__", "__Value__")
-   cat("------------------------------------------ \n ")
+  output <- summary$output
+  names(input) <- c("__Parameter__", "__Value__")
+  names(output) <- c("__Parameter__", "__Value__")
+  cat("------------------------------------------ \n ")
   cat("----Summary of the Input Parameters---- \n")
   cat("------------------------------------------ \n ")
 
@@ -498,3 +583,4 @@ plot.NPHpwr<- function(x,type=c("hazard","survival","dropout","event","censor"),
   #   graphics::mtext(paste(paste0("Statistic",1:wn,sep=":"),round(x$stat.mat[,1],3),
   #                         collapse = "  "),side=3,cex=0.7)
 }
+
